# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from .....types.annotation_public import AnnotationPublic
from .raw_client import AsyncRawAnnotationsClient, RawAnnotationsClient


class AnnotationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAnnotationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAnnotationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAnnotationsClient
        """
        return self._raw_client

    def list(
        self, project_uuid: str, span_uuid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[AnnotationPublic]:
        """
        Get annotations by functions.

        Parameters
        ----------
        project_uuid : str

        span_uuid : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AnnotationPublic]
            Successful Response

        Examples
        --------
        from mirascope import Lilypad

        client = Lilypad(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.ee.projects.spans.annotations.list(
            project_uuid="project_uuid",
            span_uuid="span_uuid",
        )
        """
        _response = self._raw_client.list(project_uuid, span_uuid, request_options=request_options)
        return _response.data


class AsyncAnnotationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAnnotationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAnnotationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAnnotationsClient
        """
        return self._raw_client

    async def list(
        self, project_uuid: str, span_uuid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[AnnotationPublic]:
        """
        Get annotations by functions.

        Parameters
        ----------
        project_uuid : str

        span_uuid : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AnnotationPublic]
            Successful Response

        Examples
        --------
        import asyncio

        from mirascope import AsyncLilypad

        client = AsyncLilypad(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.ee.projects.spans.annotations.list(
                project_uuid="project_uuid",
                span_uuid="span_uuid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project_uuid, span_uuid, request_options=request_options)
        return _response.data
