# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.comment_public import CommentPublic
from .raw_client import AsyncRawCommentsClient, RawCommentsClient


class CommentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCommentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCommentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCommentsClient
        """
        return self._raw_client

    def list(
        self, span_uuid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[CommentPublic]:
        """
        Get all comments by span.

        Parameters
        ----------
        span_uuid : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CommentPublic]
            Successful Response

        Examples
        --------
        from mirascope import Lilypad

        client = Lilypad(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )
        client.spans.comments.list(
            span_uuid="span_uuid",
        )
        """
        _response = self._raw_client.list(span_uuid, request_options=request_options)
        return _response.data


class AsyncCommentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCommentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCommentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCommentsClient
        """
        return self._raw_client

    async def list(
        self, span_uuid: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[CommentPublic]:
        """
        Get all comments by span.

        Parameters
        ----------
        span_uuid : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CommentPublic]
            Successful Response

        Examples
        --------
        import asyncio

        from mirascope import AsyncLilypad

        client = AsyncLilypad(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.spans.comments.list(
                span_uuid="span_uuid",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(span_uuid, request_options=request_options)
        return _response.data
